@@ -108,6 +108,9 @@ namespace glz
          std::memcpy(&b[ix], ",", 1);
          ++ix;
       }
+      if constexpr (is_output_streaming<B>) {
+         flush_buffer(b, ix);
+      }
    }
 
    // Only object types are supported for partial
@@ -211,6 +214,248 @@ namespace glz
       }
    };
 
+   // Runtime partial write - writes only the keys specified at runtime
+   // Outputs keys in the order they appear in the input container
+   template <class T>
+      requires(glaze_object_t<T> || reflectable<T>)
+   struct to_runtime_partial
+   {
+      template <auto Opts, class Keys, class Value, is_context Ctx, class B, class IX>
+      static void op(const Keys& keys, Value&& value, Ctx&& ctx, B&& b, IX&& ix)
+      {
+         if constexpr (!check_opening_handled(Opts)) {
+            dump<'{'>(b, ix);
+            if constexpr (Opts.prettify) {
+               ctx.indentation_level += Opts.indentation_width;
+               dump<'\n'>(b, ix);
+               dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
+            }
+         }
+
+         using V = std::remove_cvref_t<Value>;
+         static constexpr auto N = reflect<V>::size;
+
+         if constexpr (N == 0) {
+            // Empty struct - any key is unknown
+            for (const auto& key_ref : keys) {
+               (void)key_ref;
+               ctx.error = error_code::unknown_key;
+               return;
+            }
+         }
+         else {
+            static constexpr auto& HashInfo = hash_info<V>;
+
+            decltype(auto) t = [&]() -> decltype(auto) {
+               if constexpr (reflectable<V>) {
+                  return to_tie(value);
+               }
+               else {
+                  return nullptr;
+               }
+            }();
+
+            bool first = true;
+
+            for (const auto& key_ref : keys) {
+               if (bool(ctx.error)) [[unlikely]] {
+                  return;
+               }
+
+               const sv key{key_ref};
+
+               // Use hash-based lookup to find the member index
+               const auto index = decode_hash_with_size<JSON, V, HashInfo, HashInfo.type>::op(
+                  key.data(), key.data() + key.size(), key.size());
+
+               if (index >= N) {
+                  ctx.error = error_code::unknown_key;
+                  return;
+               }
+
+               // Verify the key matches (hash collision check)
+               const sv actual_key = reflect<V>::keys[index];
+               if (key != actual_key) {
+                  ctx.error = error_code::unknown_key;
+                  return;
+               }
+
+               // Write separator if not first
+               if (first) {
+                  first = false;
+               }
+               else {
+                  write_object_entry_separator<Opts>(ctx, b, ix);
+               }
+
+               // Write the quoted key at runtime
+               if constexpr (vector_like<B>) {
+                  const auto k = ix + key.size() + 4 + (Opts.prettify ? 1 : 0);
+                  if (k > b.size()) [[unlikely]] {
+                     b.resize(2 * k);
+                  }
+               }
+               dump<'"'>(b, ix);
+               dump(key, b, ix);
+               if constexpr (Opts.prettify) {
+                  dump<"\": ">(b, ix);
+               }
+               else {
+                  dump<"\":">(b, ix);
+               }
+
+               // Use visit to dispatch to correct member and serialize value
+               visit<N>(
+                  [&]<size_t I>() {
+                     using val_t = field_t<V, I>;
+                     if constexpr (reflectable<V>) {
+                        to<JSON, val_t>::template op<Opts>(get_member(value, get<I>(t)), ctx, b, ix);
+                     }
+                     else {
+                        to<JSON, val_t>::template op<Opts>(get_member(value, get<I>(reflect<V>::values)), ctx, b, ix);
+                     }
+                  },
+                  index);
+            }
+         }
+
+         if constexpr (Opts.prettify) {
+            ctx.indentation_level -= Opts.indentation_width;
+            dump<'\n'>(b, ix);
+            dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
+         }
+
+         if (not bool(ctx.error)) [[likely]] {
+            dump<'}'>(b, ix);
+         }
+      }
+   };
+
+   // Runtime exclude write - writes all keys EXCEPT those specified at runtime
+   // Outputs keys in struct definition order, skipping excluded keys
+   template <class T>
+      requires(glaze_object_t<T> || reflectable<T>)
+   struct to_runtime_exclude
+   {
+      template <auto Opts, class Keys, class Value, is_context Ctx, class B, class IX>
+      static void op(const Keys& exclude_keys, Value&& value, Ctx&& ctx, B&& b, IX&& ix)
+      {
+         if constexpr (!check_opening_handled(Opts)) {
+            dump<'{'>(b, ix);
+            if constexpr (Opts.prettify) {
+               ctx.indentation_level += Opts.indentation_width;
+               dump<'\n'>(b, ix);
+               dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
+            }
+         }
+
+         using V = std::remove_cvref_t<Value>;
+         static constexpr auto N = reflect<V>::size;
+
+         if constexpr (N == 0) {
+            // Empty struct - any exclude key is unknown
+            for (const auto& key_ref : exclude_keys) {
+               (void)key_ref;
+               ctx.error = error_code::unknown_key;
+               return;
+            }
+         }
+         else {
+            static constexpr auto& HashInfo = hash_info<V>;
+
+            // First, validate all exclude keys exist and build a bitset of excluded indices
+            std::array<bool, N> excluded{};
+            for (const auto& key_ref : exclude_keys) {
+               const sv key{key_ref};
+
+               // Use hash-based lookup to find the member index
+               const auto index = decode_hash_with_size<JSON, V, HashInfo, HashInfo.type>::op(
+                  key.data(), key.data() + key.size(), key.size());
+
+               if (index >= N) {
+                  ctx.error = error_code::unknown_key;
+                  return;
+               }
+
+               // Verify the key matches (hash collision check)
+               const sv actual_key = reflect<V>::keys[index];
+               if (key != actual_key) {
+                  ctx.error = error_code::unknown_key;
+                  return;
+               }
+
+               excluded[index] = true;
+            }
+
+            decltype(auto) t = [&]() -> decltype(auto) {
+               if constexpr (reflectable<V>) {
+                  return to_tie(value);
+               }
+               else {
+                  return nullptr;
+               }
+            }();
+
+            bool first = true;
+
+            // Iterate over all keys in struct order, skipping excluded ones
+            for_each<N>([&]<size_t I>() {
+               if (bool(ctx.error)) [[unlikely]] {
+                  return;
+               }
+
+               if (excluded[I]) {
+                  return; // Skip this key
+               }
+
+               // Write separator if not first
+               if (first) {
+                  first = false;
+               }
+               else {
+                  write_object_entry_separator<Opts>(ctx, b, ix);
+               }
+
+               // Write the quoted key
+               static constexpr sv key = reflect<V>::keys[I];
+               if constexpr (vector_like<B>) {
+                  const auto k = ix + key.size() + 4 + (Opts.prettify ? 1 : 0);
+                  if (k > b.size()) [[unlikely]] {
+                     b.resize(2 * k);
+                  }
+               }
+               dump<'"'>(b, ix);
+               dump<key>(b, ix);
+               if constexpr (Opts.prettify) {
+                  dump<"\": ">(b, ix);
+               }
+               else {
+                  dump<"\":">(b, ix);
+               }
+
+               // Serialize the value
+               using val_t = field_t<V, I>;
+               if constexpr (reflectable<V>) {
+                  to<JSON, val_t>::template op<Opts>(get_member(value, get<I>(t)), ctx, b, ix);
+               }
+               else {
+                  to<JSON, val_t>::template op<Opts>(get_member(value, get<I>(reflect<V>::values)), ctx, b, ix);
+               }
+            });
+         }
+
+         if constexpr (Opts.prettify) {
+            ctx.indentation_level -= Opts.indentation_width;
+            dump<'\n'>(b, ix);
+            dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
+         }
+
+         if (not bool(ctx.error)) [[likely]] {
+            dump<'}'>(b, ix);
+         }
+      }
+   };
+
    template <class T>
       requires(glaze_value_t<T> && !custom_write<T>)
    struct to<JSON, T>
@@ -561,15 +806,19 @@ namespace glz
                }
                // now we don't have to check writing
 
-               std::memcpy(&b[ix], "\"", 1);
-               ++ix;
+               if constexpr (not Opts.raw) {
+                  std::memcpy(&b[ix], "\"", 1);
+                  ++ix;
+               }
                if (str.size()) [[likely]] {
                   const auto n = str.size();
                   std::memcpy(&b[ix], str.data(), n);
                   ix += n;
                }
-               std::memcpy(&b[ix], "\"", 1);
-               ++ix;
+               if constexpr (not Opts.raw) {
+                  std::memcpy(&b[ix], "\"", 1);
+                  ++ix;
+               }
             }
             else {
                const sv str = [&]() -> const sv {
@@ -916,6 +1165,9 @@ namespace glz
          std::memcpy(&b[ix], ",", 1);
          ++ix;
       }
+      if constexpr (is_output_streaming<B>) {
+         flush_buffer(b, ix);
+      }
    }
 
    // "key":value pair output
@@ -1025,6 +1277,9 @@ namespace glz
                      std::memcpy(&b[ix], ",", 1);
                      ++ix;
                   }
+                  if constexpr (is_output_streaming<B>) {
+                     flush_buffer(b, ix);
+                  }
 
                   to<JSON, val_t>::template op<write_unchecked_on<Opts>()>(*it, ctx, b, ix);
                }
@@ -1116,6 +1371,9 @@ namespace glz
                         std::memcpy(&b[ix], ",", 1);
                         ++ix;
                      }
+                     if constexpr (is_output_streaming<B>) {
+                        flush_buffer(b, ix);
+                     }
 
                      to<JSON, val_t>::template op<write_unchecked_on<Opts>()>(*it, ctx, b, ix);
                   }
@@ -1391,24 +1649,21 @@ namespace glz
                      if constexpr (std::integral<id_type>) {
                         dump<"\": ">(b, ix);
                         serialize<JSON>::op<Opts>(ids_v<T>[value.index()], ctx, b, ix);
-                        if constexpr (N == 0) {
-                           dump<"\n">(b, ix);
-                        }
-                        else {
+                        if constexpr (N > 0) {
                            dump<",\n">(b, ix);
+                           dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
                         }
-                        dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
                      }
                      else {
                         dump<"\": \"">(b, ix);
                         dump_maybe_empty(ids_v<T>[value.index()], b, ix);
                         if constexpr (N == 0) {
-                           dump<"\"\n">(b, ix);
+                           dump<'"'>(b, ix);
                         }
                         else {
                            dump<"\",\n">(b, ix);
+                           dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
                         }
-                        dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
                      }
                   }
                   else {
@@ -1917,6 +2172,9 @@ namespace glz
                            std::memcpy(&b[ix], ",", 1);
                            ++ix;
                         }
+                        if constexpr (is_output_streaming<B>) {
+                           flush_buffer(b, ix);
+                        }
                      }
 
                      // MSVC requires get<I> rather than keys[I]
@@ -2101,7 +2359,9 @@ namespace glz
             ix += N;
          };
 
-         b[ix++] = '"';
+         if constexpr (not Opts.raw) {
+            b[ix++] = '"';
+         }
          write_digits.template operator()<4>(static_cast<uint64_t>(yr));
          b[ix++] = '-';
          write_digits.template operator()<2>(mo);
@@ -2130,7 +2390,9 @@ namespace glz
          }
 
          b[ix++] = 'Z';
-         b[ix++] = '"';
+         if constexpr (not Opts.raw) {
+            b[ix++] = '"';
+         }
       }
    };
 
@@ -2186,7 +2448,7 @@ namespace glz
    }
 
    template <write_supported<JSON> T, raw_buffer Buffer>
-   [[nodiscard]] glz::expected<size_t, error_ctx> write_json(T&& value, Buffer&& buffer)
+   [[nodiscard]] error_ctx write_json(T&& value, Buffer&& buffer)
    {
       return write<opts{}>(std::forward<T>(value), std::forward<Buffer>(buffer));
    }
@@ -2204,11 +2466,119 @@ namespace glz
    }
 
    template <auto& Partial, write_supported<JSON> T, raw_buffer Buffer>
-   [[nodiscard]] glz::expected<size_t, error_ctx> write_json(T&& value, Buffer&& buffer)
+   [[nodiscard]] error_ctx write_json(T&& value, Buffer&& buffer)
    {
       return write<Partial, opts{}>(std::forward<T>(value), std::forward<Buffer>(buffer));
    }
 
+   // Runtime partial write - writes only the keys specified at runtime
+   // Keys is a range of string-like types (e.g., std::vector<std::string>, std::array<std::string_view, N>)
+   // Output key order matches the order in the input container
+   // Returns error_code::unknown_key if any key doesn't exist in the struct
+
+   template <auto Opts = opts{}, class T, class Keys, output_buffer Buffer>
+      requires((glaze_object_t<T> || reflectable<T>) && range<Keys>)
+   [[nodiscard]] error_ctx write_json_partial(T&& value, const Keys& keys, Buffer&& buffer)
+   {
+      using traits = buffer_traits<std::remove_cvref_t<Buffer>>;
+
+      if constexpr (traits::is_resizable) {
+         if (buffer.size() < 2 * write_padding_bytes) {
+            buffer.resize(2 * write_padding_bytes);
+         }
+      }
+      context ctx{};
+      size_t ix = 0;
+      to_runtime_partial<std::remove_cvref_t<T>>::template op<set_json<Opts>()>(keys, std::forward<T>(value), ctx,
+                                                                                buffer, ix);
+      if (bool(ctx.error)) [[unlikely]] {
+         return {ix, ctx.error, ctx.custom_error_message};
+      }
+
+      traits::finalize(buffer, ix);
+      return {ix, error_code::none, ctx.custom_error_message};
+   }
+
+   template <auto Opts = opts{}, class T, class Keys, raw_buffer Buffer>
+      requires((glaze_object_t<T> || reflectable<T>) && range<Keys>)
+   [[nodiscard]] error_ctx write_json_partial(T&& value, const Keys& keys, Buffer&& buffer)
+   {
+      context ctx{};
+      size_t ix = 0;
+      to_runtime_partial<std::remove_cvref_t<T>>::template op<set_json<Opts>()>(keys, std::forward<T>(value), ctx,
+                                                                                buffer, ix);
+      if (bool(ctx.error)) [[unlikely]] {
+         return {ix, ctx.error, ctx.custom_error_message};
+      }
+      return {ix, error_code::none, ctx.custom_error_message};
+   }
+
+   template <auto Opts = opts{}, class T, class Keys>
+      requires((glaze_object_t<T> || reflectable<T>) && range<Keys>)
+   [[nodiscard]] glz::expected<std::string, error_ctx> write_json_partial(T&& value, const Keys& keys)
+   {
+      std::string buffer{};
+      const auto result = write_json_partial<Opts>(std::forward<T>(value), keys, buffer);
+      if (result) [[unlikely]] {
+         return glz::unexpected(static_cast<error_ctx>(result));
+      }
+      return {std::move(buffer)};
+   }
+
+   // Runtime exclude write - writes all keys EXCEPT those specified at runtime
+   // Keys is a range of string-like types (e.g., std::vector<std::string>, std::array<std::string_view, N>)
+   // Output key order matches the struct definition order
+   // Returns error_code::unknown_key if any exclude key doesn't exist in the struct
+
+   template <auto Opts = opts{}, class T, class Keys, output_buffer Buffer>
+      requires((glaze_object_t<T> || reflectable<T>) && range<Keys>)
+   [[nodiscard]] error_ctx write_json_exclude(T&& value, const Keys& exclude_keys, Buffer&& buffer)
+   {
+      using traits = buffer_traits<std::remove_cvref_t<Buffer>>;
+
+      if constexpr (traits::is_resizable) {
+         if (buffer.size() < 2 * write_padding_bytes) {
+            buffer.resize(2 * write_padding_bytes);
+         }
+      }
+      context ctx{};
+      size_t ix = 0;
+      to_runtime_exclude<std::remove_cvref_t<T>>::template op<set_json<Opts>()>(exclude_keys, std::forward<T>(value),
+                                                                                ctx, buffer, ix);
+      if (bool(ctx.error)) [[unlikely]] {
+         return {ix, ctx.error, ctx.custom_error_message};
+      }
+
+      traits::finalize(buffer, ix);
+      return {ix, error_code::none, ctx.custom_error_message};
+   }
+
+   template <auto Opts = opts{}, class T, class Keys, raw_buffer Buffer>
+      requires((glaze_object_t<T> || reflectable<T>) && range<Keys>)
+   [[nodiscard]] error_ctx write_json_exclude(T&& value, const Keys& exclude_keys, Buffer&& buffer)
+   {
+      context ctx{};
+      size_t ix = 0;
+      to_runtime_exclude<std::remove_cvref_t<T>>::template op<set_json<Opts>()>(exclude_keys, std::forward<T>(value),
+                                                                                ctx, buffer, ix);
+      if (bool(ctx.error)) [[unlikely]] {
+         return {ix, ctx.error, ctx.custom_error_message};
+      }
+      return {ix, error_code::none, ctx.custom_error_message};
+   }
+
+   template <auto Opts = opts{}, class T, class Keys>
+      requires((glaze_object_t<T> || reflectable<T>) && range<Keys>)
+   [[nodiscard]] glz::expected<std::string, error_ctx> write_json_exclude(T&& value, const Keys& exclude_keys)
+   {
+      std::string buffer{};
+      const auto result = write_json_exclude<Opts>(std::forward<T>(value), exclude_keys, buffer);
+      if (result) [[unlikely]] {
+         return glz::unexpected(static_cast<error_ctx>(result));
+      }
+      return {std::move(buffer)};
+   }
+
    template <write_supported<JSON> T, class Buffer>
    [[nodiscard]] error_ctx write_jsonc(T&& value, Buffer&& buffer)
    {
@@ -2228,7 +2598,10 @@ namespace glz
       if (bool(ec)) [[unlikely]] {
          return ec;
       }
-      return {buffer_to_file(buffer, file_name)};
+      if (auto file_ec = buffer_to_file(buffer, file_name); bool(file_ec)) {
+         return {0, file_ec};
+      }
+      return {};
    }
 }
 
